--[[ FuBar_BongosFu - FuBar support for Bongos2 ]]--

local VERSION = tonumber(("$Revision: 46125 $"):match("%d+"))

BongosFu = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceEvent-2.0", "AceConsole-2.0", "FuBarPlugin-2.0")
local BongosFu, self = BongosFu, BongosFu
BongosFu.version = "r" .. VERSION
BongosFu.revision = VERSION
BongosFu.date = ("$Date: 2007-08-09 12:46:35 -0400 (Thu, 09 Aug 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
BongosFu.hasIcon = "Interface\\Icons\\INV_Misc_Drum_04"
BongosFu.hasNoText = true
BongosFu.hasNoColor = true
BongosFu.hideWithoutStandby = true
BongosFu.cannotDetachTooltip = true
BongosFu.independentProfile = false
BongosFu.blizzardTooltip = true

BongosFu:RegisterDB("Bongos2FuDB")
BongosFu:RegisterDefaults('profile', {
   settings = {
      manageBongosProfiles = false,
      hideButtonTooltipsInCombat = false,
   },
})

local L = AceLibrary("AceLocale-2.2"):new("FuBar_BongosFu")

function BongosFu:OnEnable()
   self.bongosOptionsEnabled = select(4, GetAddOnInfo('Bongos2_Options'))
   self.wereButtonTooltipsShown = false
   self.wasUnlocked = false
   self.globalScale = 1
   self.globalAlpha = 1
   self:OnProfileDisable()
   self:OnProfileEnable()
end

function BongosFu:OnProfileDisable()
   self.manageBongosProfiles = self.db.profile.settings.manageBongosProfiles
end

function BongosFu:OnProfileEnable()
   self:SetManageBongosProfiles(self.manageBongosProfiles)
   if self:ManageBongosProfiles() then
      _, profile = self:GetProfile()
      if Bongos:MatchProfile(profile) then
         Bongos:SetProfile(profile)
      else
         Bongos:SaveProfile(profile)
      end
   end
   self:SetButtonTooltipsHiddenInCombat(self:ButtonTooltipsHiddenInCombat())
end

function BongosFu:ManageBongosProfiles()
   return self.db.profile.settings.manageBongosProfiles
end

function BongosFu:SetManageBongosProfiles(enabled)
   if enabled then
      self.db.profile.settings.manageBongosProfiles = true
   else
      self.db.profile.settings.manageBongosProfiles = false
   end
end

function BongosFu:OnClick()
   local shift, ctrl, alt = IsShiftKeyDown(), IsControlKeyDown(), IsAltKeyDown()
   if not shift and not ctrl and not alt then
      if not self.bongosOptionsEnabled then
         return
      end
      if not BongosOptions then
         LoadAddOn('Bongos2_Options')
      elseif BongosOptions:IsVisible() then
         BongosOptions:Hide()
      else
         BongosOptions:Show()
      end
   elseif shift and not ctrl and not alt then
      Bongos:ToggleLockedBars()
   elseif not shift and ctrl and not alt then
      if BongosActionConfig then
         BongosActionConfig:ShowTooltips(not BongosActionConfig:ShowingTooltips())
      end
   elseif not shift and not ctrl and alt then
      self:SetBindingsModeEnabled(not self:BindingsModeEnabled())
   end
   self:UpdateTooltip()
end

function BongosFu:OnTooltipUpdate()
   GameTooltip:AddLine("BongosFu")
   GameTooltip:AddLine(" ")
   GameTooltip:AddDoubleLine(L["Bars locked:"], (Bongos:IsLocked() and L["Yes"] or L["No"]), 1, 1, 0, 1, 1, 1)
   if BongosActionConfig then
      GameTooltip:AddDoubleLine(L["Tooltips shown:"], (BongosActionConfig:ShowingTooltips() and L["Yes"] or L["No"]), 1, 1, 0, 1, 1, 1)
      GameTooltip:AddDoubleLine(L["Key bindings mode:"], (self:BindingsModeEnabled() and L["On"] or L["Off"]), 1, 1, 0, 1, 1, 1)
   end
   GameTooltip:AddLine(" ")
   if self.bongosOptionsEnabled then
      if BongosOptions and BongosOptions:IsVisible() then
         GameTooltip:AddLine(L["|cffffff00Click|r to close Bongos Options."], 0.2, 1, 0.2)
      else
         GameTooltip:AddLine(L["|cffffff00Click|r to open Bongos Options."], 0.2, 1, 0.2)
      end
   end
   if Bongos:IsLocked() then
      GameTooltip:AddLine(L["|cffffff00Shift-Click|r to unlock bars."], 0.2, 1, 0.2)
   else
      GameTooltip:AddLine(L["|cffffff00Shift-Click|r to lock bars."], 0.2, 1, 0.2)
   end
   if BongosActionConfig then
      if BongosActionConfig:ShowingTooltips() then
         GameTooltip:AddLine(L["|cffffff00Control-Click|r to hide button tooltips."], 0.2, 1, 0.2)
      else
         GameTooltip:AddLine(L["|cffffff00Control-Click|r to show button tooltips."], 0.2, 1, 0.2)
      end
      if self:BindingsModeEnabled() then
         GameTooltip:AddLine(L["|cffffff00Alt-Click|r to disable key bindings mode."], 0.2, 1, 0.2)
      else
         GameTooltip:AddLine(L["|cffffff00Alt-Click|r to enable key bindings mode."], 0.2, 1, 0.2)
      end
   end
   GameTooltip:AddLine(L["|cffffff00Right-Click|r to change settings."], 0.2, 1, 0.2)
end

function BongosFu:ButtonTooltipsHiddenInCombat()
   return self.db.profile.settings.hideButtonTooltipsInCombat
end

function BongosFu:SetButtonTooltipsHiddenInCombat(enabled)
   if not BongosActionConfig then return end
   if enabled then
      self.db.profile.settings.hideButtonTooltipsInCombat = true
      if not self:IsEventRegistered("PLAYER_REGEN_DISABLED") then
         self:RegisterEvent("PLAYER_REGEN_DISABLED")
      end
      if not self:IsEventRegistered("PLAYER_REGEN_ENABLED") then
         self:RegisterEvent("PLAYER_REGEN_ENABLED")
      end
   else
      self.db.profile.settings.hideButtonTooltipsInCombat = false
      if self:IsEventRegistered("PLAYER_REGEN_DISABLED") then
         self:UnregisterEvent("PLAYER_REGEN_DISABLED")
      end
      if self:IsEventRegistered("PLAYER_REGEN_ENABLED") then
         self:UnregisterEvent("PLAYER_REGEN_ENABLED")
      end
   end
end

function BongosFu:PLAYER_REGEN_DISABLED()
   if BongosActionConfig and BongosActionConfig:ShowingTooltips() then
      self.wereButtonTooltipsShown = true
      BongosActionConfig:ShowTooltips(false)
   end
end

function BongosFu:PLAYER_REGEN_ENABLED()
   if BongosActionConfig and self.wereButtonTooltipsShown then
      self.wereButtonTooltipsShown = false
      BongosActionConfig:ShowTooltips(true)
   end
end

function BongosFu:BindingsModeEnabled()
   if not KeyBound then return false end
   return KeyBound:IsShown()
end

function BongosFu:SetBindingsModeEnabled(enabled)
   if not KeyBound then return end
   if enabled then
      self.wasUnlocked = not Bongos:IsLocked()
      if self.wasUnlocked then
         Bongos:SetLock(true)
      end
      KeyBound:Activate()
   else
      if self.wasUnlocked and Bongos:IsLocked() then
         Bongos:SetLock(false)
      end
      KeyBound:Deactivate()
   end
end

function self:GlobalScale()
   return self.globalScale
end

function self:SetGlobalScale(scale)
   if self.globalScale ~= scale then
      self.globalScale = scale
      BBar:ForAll("SetFrameScale", scale)
   end
end

function self:GlobalAlpha()
   return self.globalAlpha
end

function self:SetGlobalAlpha(alpha)
   if self.globalAlpha ~= alpha then
      self.globalAlpha = alpha
      BBar:ForAll("SetFrameAlpha", alpha)
   end
end

function self:Noop(...)
   return 0
end

BongosFu.OnMenuRequest = {
   type = 'group',
   handler = BongosFu,
   args = {
      lockBars = {
         order = 10,
         name = L["Lock Bar Positions"],
         desc = L["When set, Bongos bars are locked in place."],
         type = 'toggle',
         get = function() return Bongos:IsLocked() end,
         set = function(v) Bongos:ToggleLockedBars(v) end,
         disabled = "BindingsModeEnabled",
      },
      sticky = {
         order = 20,
         name = L["Enable Sticky Bars"],
         desc = L["When set, Bongos bars will stick together if placed next to one another."],
         type = 'toggle',
         get = function() return Bongos:IsSticky() end,
         set = function(v) Bongos:SetSticky(v) end,
      },
      minimapButton = {
         order = 30,
         name = L["Show Minimap Button"],
         desc = L["When set, the Bongos minimap button will be made visible."],
         type = 'toggle',
         get = function() return Bongos:ShowingMinimap() end,
         set = function(v) Bongos:SetShowMinimap(v) end,
      },
      globalScale = {
         order = 33,
         name = L["Global Scale"],
         desc = L["Controls the scale of all Bongos bars."],
         type = 'range',
         min = 0.5,
         max = 1.5,
         step = 0.01,
         isPercent = true,
         get = "GlobalScale",
         set = "SetGlobalScale"
      },
      globalAlpha = {
         order = 37,
         name = L["Global Opacity"],
         desc = L["Controls the opacity of all Bongos bars."],
         type = 'range',
         min = 0,
         max = 1,
         step = 0.01,
         isPercent = true,
         get = "GlobalAlpha",
         set = "SetGlobalAlpha"
      },
      actionBars = {
         order = 40,
         name = L["Action Bars"],
         desc = L["Settings that apply to Bongos action bars."],
         type = 'group',
         disabled = function() return not BongosActionConfig end,
         args = {
            colorRanged = {
               order = 10,
               name = L["Enable Range Coloring"],
               desc = L["When set, buttons with range-sensitive actions will be colored differently if your target is out of range."],
               type = 'toggle',
               get = (BongosActionConfig and
                      function() return BongosActionConfig:RangeColoring() end) or "Noop",
               set = (BongosActionConfig and
                      function(v) BongosActionConfig:SetRangeColoring(v) end) or "Noop",
            },
            rangeColor = {
               order = 20,
               name = L["Range Color"],
               desc = L["The color to use for range coloring of buttons."],
               type = 'color',
               get = (BongosActionConfig and
                      function() return BongosActionConfig:GetRangeColor() end) or "Noop",
               set = (BongosActionConfig and
                      function(v) BongosActionConfig:SetRangeColor(v) end) or "Noop",
               disabled = function() return not BongosActionConfig:RangeColoring() end
            },
            emptyButtons = {
               order = 30,
               name = L["Show Empty Buttons"],
               desc = L["When set, empty button slots in your action bars will be visible."],
               type = 'toggle',
               get = (BongosActionConfig and
                      function() return BongosActionConfig:ShowingEmptyButtons() end) or "Noop",
               set = (BongosActionConfig and
                      function(v) BongosActionConfig:ShowEmptyButtons(v) end) or "Noop",
            },
            buttonTooltips = {
               order = 40,
               name = L["Show Button Tooltips"],
               desc = L["When set, hovering your mouse over a button will display its tooltip."],
               type = 'toggle',
               get = (BongosActionConfig and
                      function() return BongosActionConfig:ShowingTooltips() end) or "Noop",
               set = (BongosActionConfig and
                      function(v) BongosActionConfig:ShowTooltips(v) end) or "Noop",
               disabled = InCombatLockdown
            },
            hideButtonTooltipsInCombat = {
               order = 50,
               name = L["Hide Button Tooltips In Combat"],
               desc = L["When set, button tooltips will be hidden in combat."],
               type = 'toggle',
               get = "ButtonTooltipsHiddenInCombat",
               set = "SetButtonTooltipsHiddenInCombat",
               disabled = function() return not BongosActionConfig:ShowingTooltips() or InCombatLockdown() end,
            },
            macroText = {
               order = 60,
               name = L["Show Macro Text"],
               desc = L["When set, buttons containing macros will display the macro name on the button itself."],
               type = 'toggle',
               get = (BongosActionConfig and
                      function() return BongosActionConfig:ShowingMacros() end) or "Noop",
               set = (BongosActionConfig and
                      function(v) BongosActionConfig:ShowMacros(v) end) or "Noop",
            },
            hotkeys = {
               order = 70,
               name = L["Show Hotkeys"],
               desc = L["When set, a button's key binding, if any, will be displayed on the button itself."],
               type = 'toggle',
               get = (BongosActionConfig and
                      function() return BongosActionConfig:ShowingHotkeys() end) or "Noop",
               set = (BongosActionConfig and
                      function(v) return BongosActionConfig:ShowHotkeys(v) end) or "Noop",
            },
            lockButtons = {
               order = 80,
               name = L["Lock Buttons"],
               desc = L["When set, buttons cannot be dragged off of action bars unless a 'Quick Move Key' is depressed."],
               type = 'toggle',
               get = (BongosActionConfig and
                      function() return BongosActionConfig:ButtonsLocked() end) or "Noop",
               set = (BongosActionConfig and
                      function(v) BongosActionConfig:LockButtons(v) end) or "Noop",
            },
            quickMoveKey = {
               order = 90,
               name = L["Quick Move Key"],
               desc = L["Choose which key, if any, will allow movement of buttons even when the 'Lock Buttons' option is enabled."],
               type = 'group',
               args = {
                  none = {
                     order = 10,
                     name = L["None"],
                     desc = L["When set, button movement is disabled completely when the 'Lock Buttons' option is enabled."],
                     type = 'toggle',
                     isRadio = true,
                     get = function()
                              return BongosActionConfig and
                                 BongosActionConfig:GetQuickMoveMode() == 0
                           end,
                     set = function(v)
                              return BongosActionConfig and
                                 BongosActionConfig:SetQuickMoveMode(0)
                           end,
                  },
                  shift = {
                     order = 20,
                     name = L["Shift"],
                     desc = L["When set, holding down <shift> will allow button movement even when the 'Lock Buttons' option is enabled."],
                     type = 'toggle',
                     isRadio = true,
                     get = function()
                              return BongosActionConfig and
                                 BongosActionConfig:GetQuickMoveMode() == 1
                           end,
                     set = function(v)
                              return BongosActionConfig and
                                 BongosActionConfig:SetQuickMoveMode(1)
                           end,
                  },
                  Control = {
                     order = 30,
                     name = L["Control"],
                     desc = L["When set, holding down <control> will allow button movement even when the 'Lock Buttons' option is enabled."],
                     type = 'toggle',
                     isRadio = true,
                     get = function()
                              return BongosActionConfig and
                                 BongosActionConfig:GetQuickMoveMode() == 2
                           end,
                     set = function(v)
                              return BongosActionConfig and
                                 BongosActionConfig:SetQuickMoveMode(2)
                           end,
                  },
                  Alt = {
                     order = 40,
                     name = L["Alt"],
                     desc = L["When set, holding down <alt> will allow button movement even when the 'Lock Buttons' option is enabled."],
                     type = 'toggle',
                     isRadio = true,
                     get = function()
                              return BongosActionConfig and
                                 BongosActionConfig:GetQuickMoveMode() == 3
                           end,
                     set = function(v)
                              return BongosActionConfig and
                                 BongosActionConfig:SetQuickMoveMode(3)
                           end,
                  },
               }
            },
            enableBindingsMode = {
               order = 95,
               name = L["Enable Key Bindings Mode"],
               desc = L["Enable interactive key bindings mode allowing keys to be assigned to Bongos action bar buttons."],
               type = 'toggle',
               get = "BindingsModeEnabled",
               set = "SetBindingsModeEnabled",
               disabled = InCombatLockdown
            },
         }
      },
      profiles = {
         order = 50,
         name = L["Bongos Profiles"],
         desc = L["Settings that apply to Bongos profiles."],
         type = 'group',
         args = {
            manage = {
               order = 1,
               name = L["Manage Profiles"],
               desc = L["Allows BongosFu to manage Bongos profiles based on Ace profile names."],
               type = 'toggle',
               get = "ManageBongosProfiles",
               set = "SetManageBongosProfiles"
            },
         },
      },
   }
}
